#################################################
#
# Create data for second case study in the SIENA design paper
# using NetSim
# and the intital data and empirical specifications vom Baerveldt et al. 2008, p. 573
# combined with the meta results from real data
#
# - Take 10 empirical school classes
# - Simulate two waves of data each model 50 times per class
#    - models have turnover of 0, 5, 10 %
#    - the missing rates are 0, 10, 20 %     = 3 x 3 models
# - Results in 4500 data sets to be re-estimated
# - save each simulation result as a data set that can be used in RSiena
# - Compare descriptives with empirical first wave
#    - outdegree-distribution
#    - delinquency distribution
#
# Christoph Stadtfeld
# 4 February 2015
##################################################

library(NetSim)
library(igraph)
rm(list = ls())

nSimulations <- 50

myPlot <- function(processState){
    plot(as.igraph(get_network(processState, "friendship")),
         vertex.color = as.numeric(get_attribute_container(processState, "gender")),
         vertex.size = 3 + as.numeric(get_attribute_container(processState, "delinquency")),
         vertex.label = "",
         edge.arrow.size = 0.5,
         edge.curved = 0.2
         )
}

# transform process state into a
# SIENA compatible RData object and store it
# apply the MISSINGS and TURNOVER transformations
getNetworkDelinquencyGenderForSiena <- function(processState, missings = 0, turnover = 0, wave = 1){
    net <- as.matrix(get_network(processState, "friendship"))
    del <- as.numeric(get_attribute_container(processState, "delinquency"))
    gen <- as.numeric(get_attribute_container(processState, "gender"))
    dim <- dim(net)[1]
    
    # add empty values at the end
    nTurn <- sum(turnover)
    net2 <- matrix(10, dim + nTurn, dim + nTurn)
    net2[1:dim, 1:dim] <- net
    del2 <- rep(NA, dim + nTurn)
    del2[1:dim] <- del
    gen2 <- rep(NA, dim + nTurn)
    gen2[1:dim] <- gen
    # set missings to NA.. only consider non-10 values
    net2[which(missings == 1), which(missings == 1)] <- NA
    del2[which(missings == 1)] <- NA
    #gen2[which(missings == 1)] <- NA
    # move turnover lines to the end
    # TODO also do this for the attributes
    if (wave == 2 && nTurn > 0){
        ind <- 1:dim
        tind <- which(turnover == 1)
        ind[which(turnover == 1)] <- (dim + 1): (dim + nTurn)
        ind <- c(ind, tind)
        net2 <- net2[ind, ind]
        del2 <- del2[ind]
        gen2 <- gen2[ind]
    }
    net <- net2
    del <- del2
    gen <- gen2
    return(list(net, del, gen))
    #save(file = dataPath, net, del, gen)
}

# Creating list variables
# classIds <- c(1,3,4,6) # according to baerveldth1346.csv
classIds <- c(1, 3, 4, 6, 7, 8, 9, 10, 11, 13) # 10 classes
friendship.w1 <- list()
friendship.w2 <- list()
friendship.w3 <- list()
delinquency.w1 <- list()
coethnicity <- list()
ethnicity <- list()
gender <- list()
sienaData <- list()

# Loading data
for (i in c( 1 : length(classIds) ) ){
    friendship.w1[[i]] <- as.matrix(read.table(paste0("CB_data/N34_",classIds[i],".DAT")))
    delinquency.w1[[i]] <- unlist(read.table(paste0("CB_data/cbc",classIds[i],".dat"))[,1])
    gender[[i]] <- unlist(read.table(paste0("CB_data/CBE",classIds[i],".DAT"))[,1])
    coethnicity[[i]] <- as.matrix(read.table(paste0("CB_data/cbe",classIds[i],".sim")))
    # change co-ethnicity variable to ethnicity
    ethnicity[[i]] <- clusters(graph.adjacency(coethnicity[[i]]))$membership
}

# one simulation run
specifySimulation <- function(classId){
    
    nActors <- dim(friendship.w1[[which(classIds == classId)]])[1]

    friendship.network <- create_network(friendship.w1[[which(classIds == classId)]])
    delinquency.attribute <- create_scale_attribute_container(delinquency.w1[[which(classIds == classId)]], min = 0, max = 4, by = 1)
    gender.attribute <- create_scale_attribute_container(gender[[which(classIds == classId)]], min = 1, max = 2, by = 1)
    ethnicity.attribute <- create_scale_attribute_container(ethnicity[[which(classIds == classId)]], min = 1, max = 7, by = 1)
    
    processState <- create_process_state()
    processState <- add_network(processState, friendship.network, name = "friendship")
    networkIndex <- get_network_index(processState, name = "friendship")
    processState <- add_attribute_container(processState, delinquency.attribute, name = "delinquency")
    delinquencyIndex <- get_attribute_container_index(processState,  name = "delinquency")
    processState <- add_attribute_container(processState, gender.attribute, name = "gender")
    genderIndex <- get_attribute_container_index(processState,  name = "gender")
    processState <- add_attribute_container(processState, ethnicity.attribute, name = "ethnicity")
    ethnicityIndex <- get_attribute_container_index(processState,  name = "ethnicity")
    
    modelManager <- create_model_manager()
    
    # effects for network model
    effectContainerHomophily <- create_effect_container()
    effectContainerHomophily <- add_to_effect_container(
        effectContainerHomophily,
        create_effect("density", networkIndex),
        -2.82)
    effectContainerHomophily <- add_to_effect_container(
        effectContainerHomophily,
        create_effect("recip", networkIndex),
        2.03)
    effectContainerHomophily <- add_to_effect_container(
        effectContainerHomophily,
        create_effect("transTrip", networkIndex),
        0.74)
    effectContainerHomophily <- add_to_effect_container(
        effectContainerHomophily,
        create_effect("cycle3", networkIndex),
        -0.46)
    effectContainerHomophily <- add_to_effect_container(
        effectContainerHomophily,
        create_effect("simX",
                      genderIndex,
                      networkIndex,
                      0.0), # equivalent to sameX in this two-value case
        0.46)
    effectContainerHomophily <- add_to_effect_container(
        effectContainerHomophily,
        create_effect("simX",
                      delinquencyIndex,
                      networkIndex,
                      0.5),  ## TODO: get a resonable center
        0.58)
    
    # effects for behavior model
    effectContainerInfluence <- create_effect_container()
    effectContainerInfluence <- add_to_effect_container(
        effectContainerInfluence,
        create_effect("linear",
                      delinquencyIndex),
        -0.0)
    effectContainerInfluence <- add_to_effect_container(
        effectContainerInfluence,
        create_effect("quad",
                      delinquencyIndex),
        -0.35)
    effectContainerInfluence <- add_to_effect_container(
        effectContainerInfluence,
        create_effect("avAlt",
                      delinquencyIndex,
                      networkIndex, 10/18),
        0.28)
    # TODO: add the gender x delinquency interaction
    
    
    for (i in 1:nActors ){
        poissonNet <- 5.4
        poissonBeh <- 1.27
        poissonModelNet <- create_poisson_model(poissonNet)
        poissonModelBeh <- create_poisson_model(poissonBeh)
        saomHomophilyModel <- create_multinomial_choice_network_change_model(
            i - 1,
            networkIndex,
            effectContainerHomophily
            )
        saomBehaviorModel <- create_multinomial_choice_behavior_change_model(
            i - 1,
            delinquencyIndex,
            effectContainerInfluence
            )
        tieSwapUpdater <- create_tie_swap_updater(networkIndex)
        setAttributeUpdater <- create_actor_attribute_set_updater(delinquencyIndex, i - 1)
        modelManager <<- add_time_model(modelManager,
                                        poissonModelNet)
        modelManager <<- add_change_model(modelManager,
                                          poissonModelNet,
                                          saomHomophilyModel)
        modelManager <<- add_updater(modelManager,
                                     saomHomophilyModel,
                                     tieSwapUpdater)
                                        #
        modelManager <<- add_time_model(modelManager, poissonModelBeh)
        modelManager <<- add_change_model(modelManager, poissonModelBeh, saomBehaviorModel)
        modelManager <<- add_updater(modelManager, saomBehaviorModel, setAttributeUpdater)
    }

    processState <<- processState
    modelManager <<- modelManager

} # end simulation specification function

sampleFromFrequencyTable <- function(table){
    total <- sum(table)
    rand <- runif(1)
    cum <- 0
    for (i in 1:length(table)){
        cum <- cum + table[i] / total
        if (cum >= rand) return(as.numeric(names(table)[i]))
    }
}

# set network relations of turnover actors to 0
# sample covariates from population distribution
resetTurnoverActors <- function(processState, turnover){
    net <- get_network(processState, name = "friendship")
    del <- get_attribute_container(processState, name = "delinquency")
    delVec <- as.numeric(del)
    gen <- get_attribute_container(processState, name = "gender")
    genVec <- as.numeric(gen)
    for (i in which(turnover == 1)){
        # update network
        for (j in 1:length(turnover)){
            set_tie(net, i-1,j-1, 0)
            set_tie(net, j-1, i-1, 0)
        }
        # update attributes
        del <- set_value(del, i-1, sampleFromFrequencyTable(table(delVec)))
        gen <- set_value(gen, i-1, sampleFromFrequencyTable(table(genVec)))
    }
}

# TODO clean this up
createDataSetsForSetting <- function(classId, nSimulations, pTurnover, pMissing, folderName = "data"){
    prefix <- paste0(folderName, "/t", pTurnover, "-m", pMissing, "/")
    #system(paste("mkdir", folderName))
    system(paste("mkdir", prefix, "-p"))
    suffix <- paste0("-",ifelse(classId<10, paste0("0",classId), classId))
    for(iSim in 1:nSimulations){
        id <- as.character(iSim)
        if (iSim < 100) id <- paste0("0",id)
        if (iSim < 10) id <- paste0("0",id)

        #
        processState <<- NULL
        modelManager <<- NULL
        nActors <- length(gender[[which(classIds == classId)]])
        specifySimulation(classId)

        timeSpan = 0.5
        simulator <- create_simulator(processState, modelManager, timeSpan, verbose = TRUE, debug = FALSE)
        # simulate the half interval twice
        missings <- (runif(nActors) <= pMissing) * 1
        turnover <- (runif(nActors) <= pTurnover) * 1
        dataPath <- paste0(prefix,id,suffix, ".RData")
        print(paste("Simulation", iSim, "class", classId, "t:",pTurnover,"m:", pMissing))
        # variables to be updated
        nets <- list()
        gen <- c()
        del <- c()
        vars <- getNetworkDelinquencyGenderForSiena(processState, missings, turnover, wave = 1)
        nets[[1]] <- vars[[1]]
        del <- vars[[2]]
        gen <- vars[[3]]
        simulate(simulator)
        resetTurnoverActors(processState, turnover)
        simulate(simulator)
        missings <- (runif(nActors) <= pMissing) * 1
        vars <- getNetworkDelinquencyGenderForSiena(processState, missings, turnover, wave = 2)
        nets[[2]] <- vars[[1]]
        del <- rbind(del, vars[[2]])
        gen <- rbind(gen, vars[[3]])

        # store composition change
        compChange <- list()
        for ( i in 1:nActors ){
            if (turnover[i] == 1)
                compChange[[i]] <- c(1,1.5)
            else compChange[[i]] <- c(1,2)
        }
        if (sum(turnover) > 0) for (i in 1:sum(turnover)) compChange[[nActors + i]] <- c(1.5, 2)
        # remove NAs from gender variable and transform to a vector
        gen <- unlist(apply(gen, 2, function(x) max(x, na.rm = TRUE)))
        save(compChange, nets, del, gen, file = dataPath)
    }
}

# create data sets for different settings
for (classId in classIds){
    nSimulations <- 200
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0, pMissing = 0, folderName = "data")
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0.05, pMissing = 0, folderName = "data")
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0.1, pMissing = 0, folderName = "data")
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0, pMissing = 0.05, folderName = "data")
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0.05, pMissing = 0.05, folderName = "data")
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0.1, pMissing = 0.05, folderName = "data")
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0, pMissing = 0.1, folderName = "data")
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0.05, pMissing = 0.1, folderName = "data")
    createDataSetsForSetting(classId, nSimulations, pTurnover = 0.1, pMissing = 0.1, folderName = "data")
}

