##################################################
#
# Estimating a base line SIENA model that later inspires
# the simulation of the data
#
# Replicating Chris Baerveldt's delinquency and friendship study
# Baerveldt, Völker, van Rossem (2008)
# "Revisiting Selection and Influence: An Inquiry into the Friendship Networks
# of High School Students and Their Association with Delinquency"
#
# ~ specified similar to results table p. 573, using school 1, 3, 4, 6
#
# This file is supposed to be called from a meta analysis file
# with global variable classID set to identify the class
#
# Christoph Stadtfeld
# 2. September, 2013
####################################################3

library(RSiena)
library(igraph)

# Creating list variables
classIds <- c(1,3,4,6) # according to baerveldth1346.csv
classIds <- c(1, 3, 4, 6, 7, 8, 9, 10, 11, 13) # 10 classes
#classIds <- c(1, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, 16, 18, 19, 20, 21, 22, 23) # all 19 classes

# if the variable class ID has not been set externally, use 1100
if(!exists("classID"))
    assign("classID", 1, envir = .GlobalEnv)
print (paste("Class ID = ", classID," #",which(classIds == classID)))
flush.console()

friendship.w1 <- as.matrix(read.table(paste0("CB_data/N34_",classID,".DAT")))
friendship.w2 <- as.matrix(read.table(paste0("CB_data/HN34_",classID,".DAT")))
delinquency.w1 <- unlist(read.table(paste0("CB_data/cbc",classID,".dat"))[,1])
delinquency.w2 <- unlist(read.table(paste0("CB_data/cbc",classID,".dat"))[,2])
gender <- unlist(read.table(paste0("CB_data/CBE",classID,".DAT"))[,1])
coethnicity <- as.matrix(read.table(paste0("CB_data/cbe",classID,".sim")))
nActors <- dim(friendship.w1)[1]


# plot friendship networks
myplot <- function(g){
  plot(g,
       vertex.size = 3,
       vertex.label = "",
       vertex.color = "red",
       edge.arrow.size = 0.5,
       edge.color = "black"
       )
}

myplot(graph.adjacency(friendship.w1))
myplot(graph.adjacency(friendship.w2))
   
# Create SIENA data objects
siena.data <- sienaDataCreate(
    friendship = sienaDependent(
        array(c(friendship.w1, friendship.w2), dim = c(nActors, nActors, 2)),
        type="oneMode"
        ),
    delinquency = sienaDependent(
        array(c(delinquency.w1, delinquency.w2), dim = c(nActors, 2)),
        type = "behavior"
        ),
    gender = coCovar(gender),
    coethnicity = coDyadCovar(coethnicity)
    )


# Specify SIENA model according to table 4, page 537. See reference above.
# added some parameters

# Network effects
effects <- getEffects(siena.data)
effects <- includeEffects(effects, transTrip, cycle3, name = "friendship")
effects <- includeEffects(effects, egoX, altX, sameX, name = "friendship", interaction1 = "gender")
# ignore imortance of friends within school
effects <- includeEffects(effects, egoX, altX, simX, name = "friendship", interaction1 = "delinquency")
effects <- includeEffects(effects, X, name = "friendship", interaction1 = "coethnicity")

# Delinquency effects
effects <- includeEffects(effects, effFrom, name = "delinquency", interaction1 = "gender")
effects <- includeEffects(effects, avAlt, name = "delinquency", interaction1 = "friendship")


# Estimate SIENA model
?sienaAlgorithmCreate

estimateUntilConvergence <- function(siena.algorithm = NULL, data, effects, ...){

    gain <- 0.02
    
    if(is.null(siena.algorithm))
        siena.algorithm <- sienaAlgorithmCreate(
            projname = "friendship-delinquency-schools", firstg = gain)
    
    tconv <- 10
    res <- NULL
    res.old <- res
    i <- 1
    while(max(abs(tconv)) > 0.1){
        print(paste("Max convergence value =",max(abs(tconv)),". Run estimation",i,"..."))
        flush.console()
        res <- siena07(siena.algorithm,
                                 data = data,
                                 effects = effects,
                                 prevAns = res,
                                 batch = TRUE,
                                 silent = TRUE,
                                 ...)
        newTconv <- res$tconv[which(!res$fixed)]
        if (max(abs(newTconv)) > max(abs(tconv))){
            print(paste("Max convergence got worse (",max(abs(newTconv)),").. reset result.. changing gain parameter to",gain))
            flush.console()
            gain <- gain / 2
            siena.algorithm <- sienaAlgorithmCreate(
                projname = "friendship-delinquency-schools", firstg = gain)
            # reset result
            res <- res.old
        } else{
            tconv <- newTconv
            res.old <- res
        }
        i <- i + 1
        if (i >= 10) break
    }
    return(res)
}

# algorithm is created within the loop
result <- estimateUntilConvergence(data = siena.data, effects = effects,
                                   useCluster = TRUE, nbrNodes = 4)
