#################################################
#
# Simulate SIENA model based on simulated data
# for second case study in the SIENA design paper
# using the intital data and empirical specifications vom Baerveldt et al. 2008, p. 573
#
# Several networks are combined using the multigroup option
#
# A simulation ID, a turnover rate and a missing rate need to be specified
#
# Christoph Stadtfeld
# 2 September 2013
##################################################

library("RSiena")

#if(!exists("classId", envir = .GlobalEnv))
#    classId <<- 1
if(!exists("simulationId", envir = .GlobalEnv))
    simulationId <<- 1
if(!exists("pTurnover", envir = .GlobalEnv))
    pTurnover <<- 0
if(!exists("pMissing", envir = .GlobalEnv))
    pMissing <<- 0

classIds <- c(1, 3, 4, 6, 7, 8, 9, 10, 11, 13) # 10 classes


##################################
## 1. Read data and combine networks and vectors

dataPath <- paste0("data/t",pTurnover,"-m",pMissing,"/")

network.w1 <- list()
network.w2 <- list()
gender <- list()
delinquency.w1 <- list()
delinquency.w2 <- list()
for (classId in classIds){
    index <- which(classId == classIds)
    dataFileName <- paste0(
        ifelse(simulationId < 100, ifelse(simulationId < 10, paste0("00", simulationId), paste0("0", simulationId)), simulationId),
        "-",
        ifelse(classId < 10, paste0("0", classId), classId),
        ".RData"
        )
    # load variables del, gen, nets
    load(paste0(dataPath, dataFileName))
    # store in global variables
    network.w1[[index]] <- nets[[1]]
    network.w2[[index]] <- nets[[2]]
    delinquency.w1[[index]] <- del[1,]
    delinquency.w2[[index]] <- del[2,]
    gender[[index]] <- gen
}


##################################
## 2. Define SIENA objects

sienaData <- list()
for (i in c( 1 : length(classIds) ) ){
  nActors <- dim(network.w1[[i]])[1]
  sienaData[[i]] <- sienaDataCreate(
    friendship = sienaDependent(
      array(c(network.w1[[i]], network.w2[[i]]), dim = c(nActors, nActors, 2)),
      type="oneMode"
      ),
    delinquency = sienaDependent(
      array(c(delinquency.w1[[i]], delinquency.w2[[i]]), dim = c(nActors, 2)),
      type = "behavior"
      ),
    gender = coCovar(gender[[i]])
    )
}
# Create SIENA group model
sienaDataGrouped <- sienaGroupCreate(sienaData)

#################################
# 3. Specify model

# Network effects
effects <- getEffects(sienaDataGrouped)
effects <- includeEffects(effects, transTrip, cycle3, name = "friendship")
effects <- includeEffects(effects, sameX, name = "friendship", interaction1 = "gender")
# ignore imortance of friends within school
effects <- includeEffects(effects, simX, name = "friendship", interaction1 = "delinquency")

# Delinquency effects
# effects <- includeEffects(effects, effFrom, name = "delinquency", interaction1 = "gender")
effects <- includeEffects(effects, totSim, name = "delinquency", interaction1 = "friendship")

######################
# 4. Define algorithm
alg <- sienaAlgorithmCreate(projname = "reestimation")


#####################
# 5. Estimate model until convergence
result <- NULL
maxT <- 1
nIts <- 5 # max number of iterations
while (maxT > 0.2 & nIts >= 0){
    try(
        result <<- siena07(alg, data = sienaDataGrouped, effects = effects,
                           batch = TRUE,
                           silent = TRUE,
                           useCluster = TRUE, nbrNodes = 4,
                           prevAns = result
                           )
        )
    maxT <- max(abs(result$tconv))
    nIts <- nIts - 1
    # fix extremly bad converging rate parameters to the average of the others
    #netRateIds <- 1:10
    #behRateIds <- 17:26
    #problemNetRateIds <- intersect(
    #    which(abs(result$tconv) > 98),
    #    netRateIds)
    #problemBehRateIds <- intersect(
    #    which(abs(result$tconv) > 98),
    #    behRateIds)
    #avgNetRate <- mean(result$theta[setdiff(netRateIds, problemNetRateIds)])
    #avgBehRate <- mean(result$theta[setdiff(behRateIds, problemBehRateIds)])
    #for(r in problemNetRateIds){
    #        result$theta[r] <- avgNetRate
    #        #result$fix[r] <- TRUE
    #}
    #for(r in problemBehRateIds){
    #        result$theta[r] <- avgBehRate
    #        #result$fix[r] <- TRUE
    #}
    if (nIts < 0) warning(paste("Maximimum number of iterations reached without oprimal convergence. MaxT =",maxT))
}
